from unconcealment.secret_pattern import SecretPattern


class Result:
    """ Convert object to json """
    file: str
    pattern: str
    value: str

    def __init__(self, file: str, pattern: SecretPattern, value: str):
        self.file = file
        self.pattern = str(pattern).replace('SecretPattern.', '')
        self.value = value



import enum
import re
from typing import Final, Pattern, List

URL_REGEX: Final = re.compile('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+')
HASH64_REGEX = re.compile('[a-f0-9]{64}')
HASH96_REGEX = re.compile('[a-f0-9]{96}')
HASH128_REGEX = re.compile('[a-f0-9]{128}')


class ComplexSecretPattern:
    """ Inclusion and exclusions lists of regex to trigger a finding """
    inclusions: List[Pattern]
    exclusions: List[Pattern]

    def __init__(self, inclusions: List[Pattern], exclusions: List[Pattern]):
        self.inclusions = inclusions
        self.exclusions = exclusions


class SecretPattern(enum.Enum):
    """ Regexp for secret detection """
    CREDENTIAL_FILE = ComplexSecretPattern([re.compile(".*credentials.*")], [])
    AWS_KEY = ComplexSecretPattern(
        [re.compile(".*((A3T[A-Z0-9]|AKIA|AGPA|AIDA|AROA|AIPA|ANPA|ANVA|ASIA)[A-Za-z0-9/+=]{16})")],
        [re.compile('MANPATH')])
    AWS_SECRET = ComplexSecretPattern(
        [re.compile(".*(?:AWS|aws).*([A-Za-z0-9/+=]{40})")],
        [HASH64_REGEX, HASH96_REGEX, HASH128_REGEX, URL_REGEX])
    AZURE_CLIENT_ID = ComplexSecretPattern(
        [re.compile(".*(?:AZURE|azure).*(([0-9A-Fa-f]{8}-){1,}([0-9A-Fa-f]{4}-){3,}[0-9A-Fa-f]{12}).*")],
        [HASH64_REGEX, HASH96_REGEX, HASH128_REGEX, URL_REGEX])
    NPM_TOKEN = ComplexSecretPattern([re.compile(".*(?:NPM_KEY|npm_key|NPM_TOKEN|npm_token).*(([0-9A-Fa-f]{8}-){1,}"
                                                 "([0-9A-Fa-f]{4}-){3,}[0-9A-Fa-f]{12})")], [])
    GITHUB_KEY = ComplexSecretPattern([re.compile(".*((ghu|ghs|gho|ghp)_[0-9a-zA-Z]{36}).*")], [])
    GCP_KEY = ComplexSecretPattern([re.compile(".*((?i)AIza).*")], [])
    GCP_SERVICE_ACCOUNT = ComplexSecretPattern([re.compile(".*((?i)service_account).*")], [])
    HEROKU_KEY = ComplexSecretPattern(
        [re.compile(".*(?:HEROKU|heroku).*(([0-9A-Fa-f]{8}-){1,}([0-9A-Fa-f]{4}-){3,}[0-9A-Fa-f]{12}).*")], [])
    SHOPIFY_KEY = ComplexSecretPattern([re.compile(".*((?i)(shpss|shpat|shpca|shppa)_[a-fA-F0-9]{32}).*")], [])
    PIPY_KEY = ComplexSecretPattern([re.compile(".*(pypi-AgEIcHlwaS5vcmc.*[\s]*)")], [])
    FACEBOOK_KEY = ComplexSecretPattern([], [])
    TWITTER_KEY = ComplexSecretPattern([], [])



import logging


def load_file(path: str) -> str:
    """ Load data from file """
    with open(path, 'r', encoding="utf-8") as file:
        try:
            return file.read()
        except UnicodeDecodeError:
            return ""


class Config:
    """ Config class """
    instance = {'initialized': False}

    @staticmethod
    def init(parser=None):
        """ Get singleton instance of Config """
        if Config.instance['initialized']:
            return Config.instance
        args = parser.parse_args()
        config = {
            'logLevel': args.log_level if "log_level" in args and args.log_level is not None else 'INFO',
            'file': args.file if "file" in args and args.file is not None else None,
            'directory': args.directory if "directory" in args and args.directory is not None else None,
            'data': ' '.join(args.remainder) if "remainder" in args is not None
                                                and args.remainder is not None
                                                and len(args.remainder) > 0 else None,
            'initialized': True
        }
        if config['file'] is not None:
            config['data'] = load_file(config['file'])
        logging.basicConfig(level=config['logLevel'],
                            format="%(asctime)s %(levelname)s %(threadName)s %(name)s %(message)s")
        Config.instance = config
        return Config.instance

    @staticmethod
    def file():
        """ Getter """
        return Config.instance['file']

    @staticmethod
    def log_level():
        """ Getter """
        # pylint: disable=E1136
        return Config.instance['logLevel']

    @staticmethod
    def data():
        """ Getter """
        # pylint: disable=E1136
        return Config.instance['data']

    @staticmethod
    def directory():
        """ Getter """
        # pylint: disable=E1136
        return Config.instance['directory']



import argparse
import json
import logging
import os
import sys

from unconcealment import finder
from unconcealment.config import Config, load_file
from unconcealment.result import Result
from unconcealment.secret_pattern import SecretPattern


def collect_sensitive_data(file_path: str, data: str) -> list:
    """ TODO """
    results = []
    for content in data.split('\n'):
        for secret_pattern in SecretPattern:
            secret = finder.extract_secret(content, secret_pattern)
            if secret is not None:
                results.append(Result(file_path, secret_pattern, secret))
    return results


def collect_sensitive_data_from_directory(directory_path: str):
    """ Parse the directory and collect sensitive data """
    sensitive_data = []
    # pylint: disable=W0612:
    for root, dirs, files in os.walk(directory_path):
        for file in files:
            path = os.path.join(root, file)
            sensitive_data.extend(collect_sensitive_data(path, load_file(path)))
    return sensitive_data


def convert_to_json(sensitive_data):
    """ Convert the sensitive data to json """
    return json.dumps(sensitive_data, default=lambda x: x.__dict__).replace("\"file\": null, ", '')


def main():
    """ main """
    parser = argparse.ArgumentParser(description='Detect secrets (AWS, GCP or AZURE keys, NPM tokens etc...)')
    parser.add_argument('-f', '--file', type=str, required=False, help='Input file')
    parser.add_argument('-d', '--directory', type=str, required=False, help='Input directory')
    parser.add_argument('-l', '--log-level', default=logging.INFO, type=lambda x: getattr(logging, x),
                        help="Configure the logging level.")
    parser.add_argument('remainder', nargs=argparse.REMAINDER)
    Config.init(parser)
    logging.debug("Starting the application: %s, %s", Config.log_level(), Config.data())
    sensitive_data = []
    if Config.data() is not None:
        sensitive_data = collect_sensitive_data(Config.file(), Config.data())
    elif Config.directory() is not None:
        sensitive_data = collect_sensitive_data_from_directory(Config.directory())
    if sensitive_data is None or len(sensitive_data) == 0:
        logging.info("No sensitive data found")
        sys.exit(0)
    print(convert_to_json(sensitive_data))
    sys.exit(1)


if __name__ == '__main__':
    main()



from typing import Final, Optional

from unconcealment.secret_pattern import SecretPattern

MAX_TESTED_LENGTH: Final = 5000


def extract_secret(tested: str, secret_pattern: SecretPattern) -> Optional[str]:
    """ Check if a string contains a secret using regexp"""
    if len(tested) == 0:
        return None
    result = None
    for i in range(0, len(tested), MAX_TESTED_LENGTH):
        value = tested[i:i + MAX_TESTED_LENGTH]
        for inclusion in secret_pattern.value.inclusions:
            if inclusion.match(value) is None:
                return None
            search = inclusion.search(value)
            result = (search.group(1) if search is not None and len(search.groups()) >= 1 else tested).strip()
    for exclusion in secret_pattern.value.exclusions:
        search = exclusion.search(tested)
        if search:
            return extract_secret(exclusion.sub('', tested), secret_pattern)
    return result
